#define M -DTEST
#define M1 M M M M M M M M M M
#define M2 M1 M1 M1 M1 M1 M1 M1 M1 M1 M1
#define M3 M2 M2 M2 M2 M2 M2 M2 M2 M2 M2
#define M4 M3 M3 M3 M3 M3 M3 M3 M3 M3 M3
#define M5 M4 M4 M4 M4 M4 M4 M4 M4 M4 M4
#define TEXT M5 M5 M5
TEXT
