1 ACCELERATORS
VERSION 700
LANGUAGE 5, 1
{
  "a", 3
  "a", 4, ASCII
  "a", 5, VIRTKEY
  "A", 6
  "A", 7, ASCII
  "A", 8, VIRTKEY
  "1", 9
  "1", 10, ASCII
  "1", 11, VIRTKEY
  "$", 12
  "$", 13, ASCII
  "]", 15
  "]", 16, ASCII
  "^a", 18
  "^a", 19, ASCII
  0, 37, ASCII
  0, 38, VIRTKEY
  1, 40, ASCII
  1, 41, VIRTKEY
  127, 43, ASCII
  127, 44, VIRTKEY
  128, 46, ASCII
  128, 47, VIRTKEY
  255, 49, ASCII
  255, 50, VIRTKEY
  256, 52, ASCII
  256, 53, VIRTKEY
  "^A", 66
  "^A", 67, ASCII
  54321, 70, ASCII
  54321, 71, VIRTKEY
}

2 ACCELERATORS {
  42, 0, ASCII
  42, 1, VIRTKEY
  42, 2, ASCII, NOINVERT
  42, 3, VIRTKEY, NOINVERT
  42, 4, VIRTKEY, CONTROL
  42, 5, VIRTKEY, SHIFT
  42, 6, VIRTKEY, ALT
  42, 7, VIRTKEY, NOINVERT, CONTROL
  42, 8, VIRTKEY, NOINVERT, SHIFT
  42, 9, VIRTKEY, NOINVERT, ALT
  42, 10, VIRTKEY, CONTROL, SHIFT
  42, 11, VIRTKEY, CONTROL, ALT
  42, 12, VIRTKEY, SHIFT, ALT
  42, 13, VIRTKEY, NOINVERT, CONTROL, SHIFT
  42, 14, VIRTKEY, NOINVERT, CONTROL, ALT
  42, 15, VIRTKEY, NOINVERT, SHIFT, ALT
  42, 16, VIRTKEY, CONTROL, SHIFT, ALT
  42, 17, VIRTKEY, NOINVERT, CONTROL, SHIFT, ALT
  "f", 18, ASCII
  "f", 19, VIRTKEY
  "f", 20, ASCII, NOINVERT
  "f", 21, VIRTKEY, NOINVERT
  "f", 22, VIRTKEY, CONTROL
  "f", 23, VIRTKEY, SHIFT
  "f", 24, VIRTKEY, ALT
  "f", 25, VIRTKEY, NOINVERT, CONTROL
  "f", 26, VIRTKEY, NOINVERT, SHIFT
  "f", 27, VIRTKEY, NOINVERT, ALT
  "f", 28, VIRTKEY, CONTROL, SHIFT
  "f", 29, VIRTKEY, CONTROL, ALT
  "f", 30, VIRTKEY, SHIFT, ALT
  "f", 31, VIRTKEY, NOINVERT, CONTROL, SHIFT
  "f", 32, VIRTKEY, NOINVERT, CONTROL, ALT
  "f", 33, VIRTKEY, NOINVERT, SHIFT, ALT
  "f", 34, VIRTKEY, CONTROL, SHIFT, ALT
  "f", 35, VIRTKEY, NOINVERT, CONTROL, SHIFT, ALT
  "U", 36, ASCII
  "U", 37, VIRTKEY
  "U", 38, ASCII, NOINVERT
  "U", 39, VIRTKEY, NOINVERT
  "U", 40, VIRTKEY, CONTROL
  "U", 41, VIRTKEY, SHIFT
  "U", 42, VIRTKEY, ALT
  "U", 43, VIRTKEY, NOINVERT, CONTROL
  "U", 44, VIRTKEY, NOINVERT, SHIFT
  "U", 45, VIRTKEY, NOINVERT, ALT
  "U", 46, VIRTKEY, CONTROL, SHIFT
  "U", 47, VIRTKEY, CONTROL, ALT
  "U", 48, VIRTKEY, SHIFT, ALT
  "U", 49, VIRTKEY, NOINVERT, CONTROL, SHIFT
  "U", 50, VIRTKEY, NOINVERT, CONTROL, ALT
  "U", 51, VIRTKEY, NOINVERT, SHIFT, ALT
  "U", 52, VIRTKEY, CONTROL, SHIFT, ALT
  "U", 53, VIRTKEY, NOINVERT, CONTROL, SHIFT, ALT
  "7", 54, ASCII
  "7", 55, VIRTKEY
  "7", 56, ASCII, NOINVERT
  "7", 57, VIRTKEY, NOINVERT
  "7", 58, VIRTKEY, CONTROL
  "7", 59, VIRTKEY, SHIFT
  "7", 60, VIRTKEY, ALT
  "7", 61, VIRTKEY, NOINVERT, CONTROL
  "7", 62, VIRTKEY, NOINVERT, SHIFT
  "7", 63, VIRTKEY, NOINVERT, ALT
  "7", 64, VIRTKEY, CONTROL, SHIFT
  "7", 65, VIRTKEY, CONTROL, ALT
  "7", 66, VIRTKEY, SHIFT, ALT
  "7", 67, VIRTKEY, NOINVERT, CONTROL, SHIFT
  "7", 68, VIRTKEY, NOINVERT, CONTROL, ALT
  "7", 69, VIRTKEY, NOINVERT, SHIFT, ALT
  "7", 70, VIRTKEY, CONTROL, SHIFT, ALT
  "7", 71, VIRTKEY, NOINVERT, CONTROL, SHIFT, ALT
  "^j", 72, ASCII
  "^j", 73, ASCII, NOINVERT
}

