CheckRecursion MENU {
  POPUP "A" {
    POPUP "B" {
      MENUITEM "a", 1
      MENUITEM "b", 2
      MENUITEM "c", 3
      POPUP "C" {
        POPUP "D" { POPUP "E" { POPUP "F" { POPUP "G" { POPUP "H" { POPUP "I" {
          MENUITEM "d", 57134
        }}}}}}
      }
    }
    MENUITEM "efg", 23333
  }
}


CheckFlags MENU {
  MENUITEM "a", 1, CHECKED
  MENUITEM "b", 2, GRAYED
  MENUITEM "c", 3, HELP
  MENUITEM "d", 4, INACTIVE
  MENUITEM "e", 5, MENUBARBREAK
  MENUITEM "f", 6, MENUBREAK
  MENUITEM "ad", 7, CHECKED, INACTIVE
  MENUITEM SEPARATOR
  POPUP "A", CHECKED { MENUITEM "x", 100 }
  POPUP "B", GRAYED { MENUITEM "x", 101 }
  POPUP "C", HELP { MENUITEM "x", 102 }
  POPUP "D", INACTIVE { MENUITEM "x", 103 }
  POPUP "E", MENUBARBREAK { MENUITEM "x", 104 }
  POPUP "F", MENUBREAK { MENUITEM "x", 105 }
  POPUP "G", HELP, MENUBARBREAK, GRAYED {
    POPUP "H", CHECKED, MENUBREAK, HELP, INACTIVE {
      MENUITEM SEPARATOR
      MENUITEM "x", 106, INACTIVE, MENUBARBREAK
      MENUITEM SEPARATOR
    }
  }
  MENUITEM "abcdef", 8, help, inactive, menubarbreak, checked, grayed, menubreak
}


CheckOpts MENU
CHARACTERISTICS 500
LANGUAGE 1, 1
VERSION 128
BEGIN
  POPUP "&Only separator" {
    MENUITEM SEPARATOR
  }
  POPUP "O&ther things" {
    MENUITEM "&abcde", 1
    MENUITEM "a&bcde", 2
    MENUITEM "ab&cde", 3
    MENUITEM "abc&de", 4
    MENUITEM "abcd&e", -1
  }
END

