#include "windows.h"

LANGUAGE LANG_ENGLISH, SUBLANG_ENGLISH_US
randomdat RCDATA
{
	"this is a random bit of data that means nothing\0",
	0x23a9,
	0x140e,
	194292,
}

LANGUAGE LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED
randomdat RCDATA
{
	"zhe4 shi4 yi1ge4 sui2ji1 de shu4ju4, zhe4 yi4wei4zhe shen2me\0",
	0x23a9,
	0x140e,
	194292,
}

LANGUAGE LANG_GERMAN, SUBLANG_GERMAN_LUXEMBOURG
randomdat RCDATA
{
	"Dies ist ein zufälliges Bit von Daten, die nichts bedeutet\0",
	0x23a9,
	0x140e,
	194292,
}

LANGUAGE LANG_CHINESE, SUBLANG_CHINESE_SIMPLIFIED
myaccelerators ACCELERATORS
{
	"^C", 999, VIRTKEY, ALT
	"D", 1100, VIRTKEY, CONTROL, SHIFT
	"^R", 444, ASCII, NOINVERT
}
