1 VERSIONINFO
FILEVERSION 1, 2, 3, 4
PRODUCTVERSION 5, 6, 7, 8
FILEFLAGSMASK 50
FILEFLAGS 555
FILEOS 110
FILETYPE 555555
FILESUBTYPE 14
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        {
            VALUE "CompanyName", 32768
            VALUE "FileDescription", 65536
        }
    END
END
