1 DIALOGEX 1, 2, 3, 4 {}
2 DIALOG 1, 2, 3, 4 {}

3 DIALOGEX 10000, 20000, 30000, 32767
CAPTION "My Caption" {}
4 DIALOG -32768, 32767, 0, 32767
CAPTION "My Caption" {}

5 DIALOGEX 0, 1, 0, 9
STYLE 0x01234567 {}
6 DIALOG 0, 1, 0, 9
STYLE 0x01234567 {}

7 DIALOGEX 8, 7, 6, 5
FONT 50, "a" {}
8 DIALOG 8, 7, 6, 5
FONT 50, "aaaa" {}
9 DIALOGEX 8, 7, 6, 5
FONT 50, L"a" {}
10 DIALOG 8, 7, 6, 5
FONT 50, L"aaaa" {}
11 DIALOGEX 1, 2, 3, 4
FONT 50, "FONT", 51 {}
12 DIALOGEX 1, 2, 3, 4
FONT 50, "FONT", 51, 52 {}
13 DIALOGEX 1, 2, 3, 4
FONT 50, "FONT", 51, 52, 53 {}

14 DIALOGEX 1, 1, 1, 1
CAPTION "CAPTION" FONT 42, "FONT" {}
15 DIALOG 1, 1, 1, 1
CAPTION "CAPTION" FONT 42, "FONT" {}

16 DIALOGEX 2, 2, 2, 2
CAPTION "CAPTION" FONT 42, "FONT" STYLE 0 {}
17 DIALOG 2, 2, 2, 2
CAPTION "CAPTION" FONT 42, "FONT" STYLE 0 {}

18 DIALOGEX 3, 3, 3, 3
CAPTION "CAPTION" STYLE 0 {}
19 DIALOG 3, 3, 3, 3
CAPTION "CAPTION" STYLE 0 {}

20 DIALOGEX 4, 4, 4, 4 STYLE 0xFF00FF00 {}
21 DIALOG 4, 4, 4, 4 STYLE 0xFF00FF00 {}
22 DIALOGEX 4, 4, 4, 4 STYLE 0x00FF00FF {}
23 DIALOG 4, 4, 4, 4 STYLE 0x00FF00FF {}

24 DIALOGEX 5, 5, 5, 5 CAPTION "" STYLE 0 {}
25 DIALOG 5, 5, 5, 5 CAPTION "" STYLE 0 {}

26 DIALOGEX 1, 2, 3, 4, 5 {}

27 DIALOGEX 5, 5, 5, 5 CLASS "Foobar" {}
28 DIALOG 5, 5, 5, 5 CLASS 42 {}

29 DIALOGEX 0, 1, 0, 9 EXSTYLE 0x01234567 {}
30 DIALOG 0, 1, 0, 9 EXSTYLE 0x01234567 {}

31 DIALOGEX 0, 1, 0, 9 STYLE 0x76543210 EXSTYLE 0x01234567 {}
32 DIALOG 0, 1, 0, 9 EXSTYLE 0x0123456 STYLE 0x76543210 {}
