1 CURSOR PRELOAD "cursor.cur"
2 CURSOR LOADONCALL "cursor.cur"
3 CURSOR FIXED "cursor.cur"
4 CURSOR MOVEABLE "cursor.cur"
5 CURSOR DISCARDABLE "cursor.cur"
6 CURSOR PURE "cursor.cur"
7 CURSOR IMPURE "cursor.cur"
8 CURSOR SHARED "cursor.cur"
9 CURSOR NONSHARED "cursor.cur"
10 ICON PRELOAD "icon-old.ico"
11 ICON LOADONCALL "icon-old.ico"
12 ICON FIXED "icon-old.ico"
13 ICON MOVEABLE "icon-old.ico"
14 ICON DISCARDABLE "icon-old.ico"
15 ICON PURE "icon-old.ico"
16 ICON IMPURE "icon-old.ico"
17 ICON SHARED "icon-old.ico"
18 ICON NONSHARED "icon-old.ico"
19 BITMAP PRELOAD "bitmap.bmp"
20 BITMAP LOADONCALL "bitmap.bmp"
21 BITMAP FIXED "bitmap.bmp"
22 BITMAP MOVEABLE "bitmap.bmp"
23 BITMAP DISCARDABLE "bitmap.bmp"
24 BITMAP PURE "bitmap.bmp"
25 BITMAP IMPURE "bitmap.bmp"
26 BITMAP SHARED "bitmap.bmp"
27 BITMAP NONSHARED "bitmap.bmp"
28 BITMAP FIXED IMPURE "bitmap.bmp"
29 BITMAP DISCARDABLE FIXED IMPURE "bitmap.bmp"
30 BITMAP DISCARDABLE FIXED "bitmap.bmp"
31 BITMAP DISCARDABLE IMPURE "bitmap.bmp"
32 BITMAP DISCARDABLE LOADONCALL "bitmap.bmp"
33 BITMAP FIXED SHARED "bitmap.bmp"
34 BITMAP FIXED IMPURE SHARED "bitmap.bmp"
35 BITMAP FIXED IMPURE DISCARDABLE "bitmap.bmp"
