// RUN: %clang_cc1 -emit-llvm %s  -o /dev/null

  char      auto_kibitz_list[100][20] = {
                                      {"diepx"},
                                      {"ferret"},
                                      {"knightc"},
                                      {"knightcap"}};

