#include <complex.h>

int main() {
  float complex complex_float = -1.5f + -2.5f * I;
  double complex complex_double = -1.5 + -2.5 * I;
  long double complex complex_long_double = -1.5L + -2.5L * I;
  return 0; // break here
}
